import numpy as np
from matplotlib import pyplot as plt
from datetime import datetime
import os, calendar

event_type=['Rain','Thunderstorm','Snow','Fog']
num_events=len(event_type)
def date2int(string):
    date=datetime.strptime(string,'%Y-%m-%d')
    return date.toordinal()

def event2int(e):
    return event_type.index(e)

def read(filename):
    t=np.dtype([('time',np.int),('max',np.float),('mean',np.float),('min',np.float),('event','S20')])
    data=np.loadtxt(filename,delimiter=',',usecols=(0,1,2,3,21),dtype=t,skiprows=1,converters={0:date2int})
    return data

def plot_temp(dates,mean_t,max_t,min_t):
    f=plt.figure()
    year_start=datetime(2012,1,1)
    ax1=f.add_subplot(3,1,1)
    ax2=f.add_subplot(3,1,2)
    ax3=f.add_subplot(3,1,3)
    days=[(d-year_start).days+1 for d in dates]
   
    ax1.plot(days,mean_t,marker='o',color='g')
    ax2.plot(days,min_t,marker='o',color='b')
    ax3.plot(days,max_t,marker='o',color='r')
    ax1.set_title('Mean temp')
    ax1.set_xlabel('day')
    ax1.set_ylabel('Mean temp')

    ax2.set_title('Min temp')
    ax2.set_xlabel('day')
    ax2.set_ylabel('Mean temp')

    ax3.set_title('Max temp')
    ax3.set_xlabel('day')
    ax3.set_ylabel('Mean temp')

    f.subplots_adjust(hspace=0.5)
    f.savefig('plot.png')
    plt.show()
    
def plot_temp_fitted(dates,mean_t,max_t,min_t):
    f=plt.figure()
    year_start=datetime(2012,1,1)
    ax1=f.add_subplot(3,1,1)
    ax2=f.add_subplot(3,1,2)
    ax3=f.add_subplot(3,1,3)
    days=np.array([(d-year_start).days+1 for d in dates])
    
    ax1.plot(days,mean_t,marker='o',color='g')
    fit=np.polyfit(days,mean_t,1)
    ideal_temps=np.poly1d(fit)
    ax1.plot(days,ideal_temps(days),color='y',linestyle='--',label='ideal')
    ax1.legend()
    
    ax2.plot(days,min_t,marker='o',color='b')
    slope,inter=np.polyfit(days,min_t,1)
    slope, inter = np.polyfit(days, min_t, 1)
    p=np.polyfit(days, min_t, 1)
    ideal_temps = np.poly1d(p)
    ax2.plot(days,ideal_temps(days),color='y',linestyle='--',label='ideal')
    ax2.legend()
 
    ax3.plot(days,max_t,marker='o',color='r')
    p=np.polyfit(days, max_t, 1)
    ideal_temps = np.poly1d(p)
    ax3.plot(days,ideal_temps(days),color='y',linestyle='--',label='ideal')
    ax3.legend()
    ax1.set_title('Mean temp')
    ax1.set_xlabel('day')
    ax1.set_ylabel('Mean temp')

    ax2.set_title('Min temp')
    ax2.set_xlabel('day')
    ax2.set_ylabel('Mean temp')

    ax3.set_title('Max temp')
    ax3.set_xlabel('day')
    ax3.set_ylabel('Mean temp')

    f.subplots_adjust(hspace=0.5)
    f.savefig('plot_fitted.png')
    plt.show()
    


def hist(event,date):
    event_bar = [[] for i in xrange(num_events)]
    month_set=set([])
    #f=plt.figure()
    for date, event_str in zip(dates, event):
        if len(event_str) == 0:
            continue  # Skip blank events
        month=date.month
        month_set.add(month)
        event_code=event2int(event_str)
        event_bar[event_code].append(month)
    f=plt.figure()
    ax=f.add_subplot(1,1,1)
    ax.set_title('Weather events')
    ax.set_xlabel('Month')
    ax.set_ylabel('Event Count')

    ax.hist(event_bar,label=event_type,bins=np.arange(1,7))
    month_name=list(month_set)
    month_name=[calendar.month_name[m] for m in month_set]
    ax.set_xticks(np.arange(1,7)+0.5)
    ax.set_xticklabels(month_name)
   
    ax.legend()
    plt.show()


filename=os.path.join('data','weather_data.csv')
data=read(filename)
min_t=data['min']
max_t=data['max']
mean_t=data['mean']
dates=[datetime.fromordinal(d) for d in data['time']]
events=data['event']

plot_temp(dates,mean_t,max_t,min_t)
plot_temp_fitted(dates,mean_t,max_t,min_t)
hist(events,dates)

